@@ -15,27 +15,32 @@
 ## along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
 ##
 
-import urllib2
+import urllib.request as urllib2
 import socket
 import re
+import ssl
 
-from common import helpers
-from common import gajim
+from gajim.common import app
+from gajim.common import helpers
 import logging
 
-if gajim.HAVE_PYCURL:
+import os
+if os.name == 'nt':
+    import certifi
+
+if app.HAVE_PYCURL:
     import pycurl
-    from cStringIO import StringIO
+    from io import StringIO
 
 
 log = logging.getLogger('gajim.plugin_system.url_image_preview.http_functions')
 
-def get_http_head(account, url):
+def get_http_head(account, url, verify):
     # Check if proxy is used
     proxy = helpers.get_proxy_info(account)
     if proxy and proxy['type'] in ('http', 'socks5'):
         return _get_http_head_proxy(url, proxy)
-    return _get_http_head_direct(url)
+    return _get_http_head_direct(url, verify)
 
 def get_http_file(account, attrs):
     # Check if proxy is used
@@ -45,34 +50,38 @@ def get_http_file(account, attrs):
     else:
         return _get_http_direct(attrs)
 
-def _get_http_head_direct(url):
+def _get_http_head_direct(url, verify):
     log.debug('Head request direct for URL: %s' % url)
     try:
         req = urllib2.Request(url)
         req.get_method = lambda: 'HEAD'
-        req.add_header('User-Agent', 'Gajim %s' % gajim.version)
-        f = urllib2.urlopen(req)
-    except Exception, ex:
+        req.add_header('User-Agent', 'Gajim %s' % app.version)
+        if not verify:
+            context = ssl.create_default_context()
+            context.check_hostname = False
+            context.verify_mode = ssl.CERT_NONE
+            log.warning('CERT Verification disabled')
+            f = urllib2.urlopen(req, timeout=30, context=context)
+        else:
+            if os.name == 'nt':
+                f = urllib2.urlopen(req, cafile=certifi.where())
+            else:
+                f = urllib2.urlopen(req)
+    except Exception as ex:
         log.debug('Could not get head response for URL: %s' % url)
         log.debug("%s" % str(ex))
         return ('', 0)
-    url_headers = f.info()
-    ctype = ''
-    ctype_list = url_headers.getheaders('Content-Type')
-    if ctype_list:
-        ctype = ctype_list[0]
-    clen = 0
-    clen_list = url_headers.getheaders('Content-Length')
-    if clen_list:
-        try:
-            clen = int(clen_list[0])
-        except ValueError:
-            pass
+    ctype = f.headers['Content-Type']
+    clen = f.headers['Content-Length']
+    try:
+        clen = int(clen)
+    except (TypeError, ValueError):
+        pass
     return (ctype, clen)
 
 def _get_http_head_proxy(url, proxy):
     log.debug('Head request with proxy for URL: %s' % url)
-    if not gajim.HAVE_PYCURL:
+    if not app.HAVE_PYCURL:
         log.error('PYCURL not installed')
         return ('', 0)
 
@@ -89,7 +98,7 @@ def _get_http_head_proxy(url, proxy):
 
         c.setopt(pycurl.MAXFILESIZE, 2000000)
         c.setopt(pycurl.WRITEFUNCTION, b.write)
-        c.setopt(pycurl.USERAGENT, 'Gajim ' + gajim.version)
+        c.setopt(pycurl.USERAGENT, 'Gajim ' + app.version)
 
         # set proxy
         c.setopt(pycurl.PROXY, proxy['host'].encode('utf-8'))
@@ -105,7 +114,7 @@ def _get_http_head_proxy(url, proxy):
         x = c.perform()
         c.close()
         headers = b.getvalue()
-    except pycurl.error, ex:
+    except pycurl.error as ex:
         log.debug('Could not get head response for URL: %s' % url)
         log.debug("%s" % str(ex))
         return ('', 0)
@@ -129,14 +138,24 @@ def _get_http_direct(attrs):
     be launched in a separated thread.
     """
     log.debug('Get request direct for URL: %s' % attrs['src'])
-    mem, alt, max_size = '', '', 2 * 1024 * 1024
+    mem, alt, max_size = b'', '', 2 * 1024 * 1024
     if 'max_size' in attrs:
         max_size = attrs['max_size']
     try:
         req = urllib2.Request(attrs['src'])
-        req.add_header('User-Agent', 'Gajim ' + gajim.version)
-        f = urllib2.urlopen(req)
-    except Exception, ex:
+        req.add_header('User-Agent', 'Gajim ' + app.version)
+        if not attrs['verify']:
+            context = ssl.create_default_context()
+            context.check_hostname = False
+            context.verify_mode = ssl.CERT_NONE
+            log.warning('CERT Verification disabled')
+            f = urllib2.urlopen(req, timeout=30, context=context)
+        else:
+            if os.name == 'nt':
+                f = urllib2.urlopen(req, cafile=certifi.where())
+            else:
+                f = urllib2.urlopen(req)
+    except Exception as ex:
         log.debug('Error loading file %s '
                     % attrs['src'] + str(ex))
         pixbuf = None
@@ -145,7 +164,7 @@ def _get_http_direct(attrs):
         while True:
             try:
                 temp = f.read(100)
-            except socket.timeout, ex:
+            except socket.timeout as ex:
                 log.debug('Timeout loading image %s '
                             % attrs['src'] + str(ex))
                 alt = attrs.get('alt', '')
@@ -172,7 +191,7 @@ def _get_http_proxy(attrs, proxy):
     separated thread.
     """
     log.debug('Get request with proxy for URL: %s' % attrs['src'])
-    if not gajim.HAVE_PYCURL:
+    if not app.HAVE_PYCURL:
         log.error('PYCURL not installed')
         return '', _('PyCURL is not installed')
     mem, alt, max_size = '', '', 2 * 1024 * 1024
@@ -185,7 +204,7 @@ def _get_http_proxy(attrs, proxy):
         c.setopt(pycurl.FOLLOWLOCATION, 1)
         c.setopt(pycurl.MAXFILESIZE, max_size)
         c.setopt(pycurl.WRITEFUNCTION, b.write)
-        c.setopt(pycurl.USERAGENT, 'Gajim ' + gajim.version)
+        c.setopt(pycurl.USERAGENT, 'Gajim ' + app.version)
         # set proxy
         c.setopt(pycurl.PROXY, proxy['host'].encode('utf-8'))
         c.setopt(pycurl.PROXYPORT, proxy['port'])
@@ -201,7 +220,7 @@ def _get_http_proxy(attrs, proxy):
         c.close()
         t = b.getvalue()
         return (t, attrs.get('alt', ''))
-    except pycurl.error, ex:
+    except pycurl.error as ex:
         alt = attrs.get('alt', '')
         if alt:
             alt += '\n'
@@ -211,7 +230,7 @@ def _get_http_proxy(attrs, proxy):
             alt += _('Timeout loading image')
         else:
             alt += _('Error loading image')
-    except Exception, ex:
+    except Exception as ex:
         log.debug('Error loading file %s ' % attrs['src'] + str(ex))
         pixbuf = None
         alt = attrs.get('alt', 'Broken image')
