@@ -15,38 +15,35 @@
 ## along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
 ##
 
-import gtk
-import gobject
 import os
 import hashlib
 import binascii
-from urlparse import urlparse
+import logging
+from urllib.parse import urlparse
 from io import BytesIO
 import shutil
+from functools import partial
 
-import logging
-import nbxmpp
-from common import gajim
-from common import ged
-from common import helpers
-from common import configpaths
-import dialogs
-from plugins import GajimPlugin
-from plugins.helpers import log_calls
-from plugins.gui import GajimPluginConfigDialog
-from conversation_textview import TextViewImage
-from .http_functions import get_http_head, get_http_file
+from gi.repository import Gtk, Gdk, GLib, GdkPixbuf
 
-from common import demandimport
-demandimport.enable()
-demandimport.ignore += ['_imp', 'PIL']
+from gajim.common import app
+from gajim.common import helpers
+from gajim.common import configpaths
+from gajim import dialogs
+from gajim.plugins import GajimPlugin
+from gajim.plugins.helpers import log_calls
+from url_image_preview.http_functions import get_http_head, get_http_file
+from url_image_preview.config_dialog import UrlImagePreviewConfigDialog
+from url_image_preview.resize_gif import resize_gif
 
-log = logging.getLogger('gajim.plugin_system.url_image_preview')
+log = logging.getLogger('gajim.plugin_system.preview')
 
+PILLOW_AVAILABLE = True
 try:
     from PIL import Image
 except:
     log.debug('Pillow not available')
+    PILLOW_AVAILABLE = False
 
 try:
     if os.name == 'nt':
@@ -57,10 +54,10 @@ try:
     from cryptography.hazmat.primitives.ciphers import algorithms
     from cryptography.hazmat.primitives.ciphers.modes import GCM
     decryption_available = True
-except Exception as e:
+except Exception:
     DEP_MSG = 'For preview of encrypted images, ' \
               'please install python-cryptography!'
-    log.debug('Cryptography Import Error: ' + str(e))
+    log.exception('Error')
     log.info('Decryption/Encryption disabled due to errors')
     decryption_available = False
 
@@ -73,33 +70,21 @@ class UrlImagePreviewPlugin(GajimPlugin):
     def init(self):
         if not decryption_available:
             self.available_text = DEP_MSG
-        self.config_dialog = UrlImagePreviewPluginConfigDialog(self)
-        self.events_handlers = {}
-        self.events_handlers['message-received'] = (
-            ged.PRECORE, self.handle_message_received)
+        self.config_dialog = partial(UrlImagePreviewConfigDialog, self)
         self.gui_extension_points = {
             'chat_control_base': (self.connect_with_chat_control,
                                   self.disconnect_from_chat_control),
-            'print_special_text': (self.print_special_text, None), }
+            'history_window':
+                (self.connect_with_history, self.disconnect_from_history),
+            'print_real_text': (self.print_real_text, None), }
         self.config_default_values = {
             'PREVIEW_SIZE': (150, 'Preview size(10-512)'),
             'MAX_FILE_SIZE': (524288, 'Max file size for image preview'),
-            'LEFTCLICK_ACTION': ('open_menuitem', 'Open')}
+            'LEFTCLICK_ACTION': ('open_menuitem', 'Open'),
+            'ANONYMOUS_MUC': (False, ''),
+            'VERIFY': (True, ''),}
         self.controls = {}
-
-    # remove oob tag if oob url == message text
-    def handle_message_received(self, event):
-        oob_node = event.stanza.getTag('x', namespace=nbxmpp.NS_X_OOB)
-        oob_url = None
-        oob_desc = None
-        if oob_node:
-            oob_url = oob_node.getTagData('url')
-            oob_desc = oob_node.getTagData('desc')
-            if oob_url and oob_url == event.msgtxt and \
-                    (not oob_desc or oob_desc == ""):
-                log.debug("Detected oob tag containing same"
-                          "url as the message text, deleting oob tag...")
-                event.stanza.delChild(oob_node)
+        self.history_window_control = None
 
     @log_calls('UrlImagePreviewPlugin')
     def connect_with_chat_control(self, chat_control):
@@ -107,31 +92,49 @@ class UrlImagePreviewPlugin(GajimPlugin):
         jid = chat_control.contact.jid
         if account not in self.controls:
             self.controls[account] = {}
-        self.controls[account][jid] = Base(self, chat_control)
+        self.controls[account][jid] = Base(self, chat_control.conv_textview)
 
     @log_calls('UrlImagePreviewPlugin')
     def disconnect_from_chat_control(self, chat_control):
         account = chat_control.contact.account.name
         jid = chat_control.contact.jid
-        self.controls[account][jid].deinit()
+        self.controls[account][jid].deinit_handlers()
         del self.controls[account][jid]
 
-    def print_special_text(self, tv, special_text, other_tags, graphics=True,
-                           iter_=None):
+    @log_calls('UrlImagePreviewPlugin')
+    def connect_with_history(self, history_window):
+        if self.history_window_control:
+            log.error("connect_with_history: deinit handlers")
+            self.history_window_control.deinit_handlers()
+        log.error("connect_with_history: create base")
+        self.history_window_control = Base(
+            self, history_window.history_textview)
+
+    @log_calls('UrlImagePreviewPlugin')
+    def disconnect_from_history(self, history_window):
+        if self.history_window_control:
+            self.history_window_control.deinit_handlers()
+        self.history_window_control = None
+
+    def print_real_text(self, tv, real_text, text_tags, graphics,
+                        iter_, additional_data):
+        if tv.used_in_history_window and self.history_window_control:
+            self.history_window_control.print_real_text(
+                real_text, text_tags, graphics, iter_, additional_data)
+
         account = tv.account
         for jid in self.controls[account]:
-            if self.controls[account][jid].chat_control.conv_textview != tv:
+            if self.controls[account][jid].textview != tv:
                 continue
-            self.controls[account][jid].print_special_text(
-                special_text, other_tags, graphics=True, iter_=iter_)
+            self.controls[account][jid].print_real_text(
+                real_text, text_tags, graphics, iter_, additional_data)
             return
 
 
 class Base(object):
-    def __init__(self, plugin, chat_control):
+    def __init__(self, plugin, textview):
         self.plugin = plugin
-        self.chat_control = chat_control
-        self.textview = self.chat_control.conv_textview
+        self.textview = textview
         self.handlers = {}
 
         self.directory = os.path.join(configpaths.gajimpaths['MY_DATA'],
@@ -142,11 +145,11 @@ class Base(object):
         try:
             self._create_path(self.directory)
             self._create_path(self.thumbpath)
-        except Exception as e:
+        except Exception:
             log.error("Error creating download and/or thumbnail folder!")
             raise
 
-    def deinit(self):
+    def deinit_handlers(self):
         # remove all register handlers on wigets, created by self.xml
         # to prevent circular references among objects
         for i in list(self.handlers.keys()):
@@ -154,20 +157,26 @@ class Base(object):
                 self.handlers[i].disconnect(i)
             del self.handlers[i]
 
-    def print_special_text(self, special_text, other_tags, graphics=True,
-                           iter_=None):
-        # remove qip bbcode
-        special_text = special_text.rsplit('[/img]')[0]
+    def print_real_text(self, real_text, text_tags, graphics, iter_,
+                        additional_data):
+        urlparts = urlparse(real_text)
+        if (urlparts.scheme not in ["https", "aesgcm"] or
+                not urlparts.netloc):
+            log.info("Not accepting URL scheme '%s' for image preview: %s",
+                     urlparts.scheme, real_text)
+            return
 
-        if special_text.startswith('www.'):
-            special_text = 'http://' + special_text
-        if special_text.startswith('ftp.'):
-            special_text = 'ftp://' + special_text
+        try:
+            oob_url = additional_data["gajim"]["oob_url"]
+        except (KeyError, AttributeError):
+            oob_url = None
 
-        urlparts = urlparse(special_text)
-        if urlparts.scheme not in ["https", "http", "ftp", "ftps", 'aesgcm'] or \
-                not urlparts.netloc:
-            log.info("Not accepting URL for image preview: %s" % special_text)
+        # allow aesgcm uris without oob marker (aesgcm uris are always
+        # httpupload filetransfers)
+        if urlparts.scheme != "aesgcm" and real_text != oob_url:
+            log.info("Not accepting URL for image preview "
+                     "(wrong or no oob data): %s", real_text)
+            log.debug("additional_data: %s", additional_data)
             return
 
         # Don't print the URL in the message window (in the calling function)
@@ -180,14 +189,14 @@ class Base(object):
         # Show URL, until image is loaded (if ever)
         ttt = buffer_.get_tag_table()
         repl_start = buffer_.create_mark(None, iter_, True)
-        buffer_.insert_with_tags(iter_, special_text,
+        buffer_.insert_with_tags(iter_, real_text,
             *[(ttt.lookup(t) if isinstance(t, str) else t) for t in ["url"]])
         repl_end = buffer_.create_mark(None, iter_, True)
 
         filename = os.path.basename(urlparts.path)
         ext = os.path.splitext(filename)[1]
         name = os.path.splitext(filename)[0]
-        namehash = hashlib.sha1(special_text).hexdigest()
+        namehash = hashlib.sha1(real_text.encode('utf-8')).hexdigest()
         newfilename = name + '_' + namehash + ext
         thumbfilename = name + '_' + namehash + '_thumb_' \
             + str(self.plugin.config['PREVIEW_SIZE']) + ext
@@ -215,18 +224,17 @@ class Base(object):
         if os.path.exists(filepath) and not os.path.exists(thumbpath):
             with open(filepath, 'rb') as f:
                 mem = f.read()
-                f.closed
-            gajim.thread_interface(
-                self._save_thumbnail, [thumbpath, (mem, '')],
-                self._update_img, [special_text, repl_start,
+            app.thread_interface(
+                self._save_thumbnail, [thumbpath, mem],
+                self._update_img, [real_text, repl_start,
                                    repl_end, filepath, encrypted])
 
         # display thumbnail if already downloadeded
         # (but only if file also exists)
         elif os.path.exists(filepath) and os.path.exists(thumbpath):
-            gajim.thread_interface(
+            app.thread_interface(
                 self._load_thumbnail, [thumbpath],
-                self._update_img, [special_text, repl_start,
+                self._update_img, [real_text, repl_start,
                                    repl_end, filepath, encrypted])
 
         # or download file, calculate thumbnail and finally display it
@@ -238,162 +246,172 @@ class Base(object):
                 # which does not fetch data, just headers
                 # then check the mime type and filesize
                 if urlparts.scheme == 'aesgcm':
-                    special_text = 'https://' + special_text[9:]
-                gajim.thread_interface(
-                    get_http_head, [self.textview.account, special_text],
-                    self._check_mime_size, [special_text, repl_start, repl_end,
+                    real_text = 'https://' + real_text[9:]
+                verify = self.plugin.config['VERIFY']
+                app.thread_interface(
+                    get_http_head, [self.textview.account, real_text, verify],
+                    self._check_mime_size, [real_text, repl_start, repl_end,
                                             filepaths, key, iv, encrypted])
 
-    def _save_thumbnail(self, thumbpath, (mem, alt)):
+    def _save_thumbnail(self, thumbpath, mem):
         size = self.plugin.config['PREVIEW_SIZE']
-        use_gtk = False
-        output = None
 
         try:
-            output = BytesIO()
-            im = Image.open(BytesIO(mem))
-            im.thumbnail((size, size), Image.ANTIALIAS)
-            im.save(output, "jpeg", quality=100, optimize=True)
-        except Exception as e:
-            if output:
-                output.close()
-            log.info("Failed to load image using pillow, "
-                     "falling back to gdk pixbuf.")
-            log.debug(e)
-            use_gtk = True
-
-        if use_gtk:
-            log.info("Pillow not available or file corrupt, "
-                     "trying to load using gdk pixbuf.")
-            try:
-                output = BytesIO()
-                loader = gtk.gdk.PixbufLoader()
-                loader.write(mem)
-                loader.close()
+            loader = GdkPixbuf.PixbufLoader()
+            loader.write(mem)
+            loader.close()
+            if loader.get_format().get_name() == 'gif':
+                pixbuf = loader.get_animation()
+            else:
                 pixbuf = loader.get_pixbuf()
-                pixbuf, w, h = self._get_pixbuf_of_size(pixbuf, size)
+        except GLib.GError as error:
+            log.info('Failed to load image using Gdk.Pixbuf')
+            log.debug(error)
 
-                def cb(buf, data=None):
-                    output.write(buf)
-                    return True
+            if not PILLOW_AVAILABLE:
+                log.info('Pillow not available')
+                return
+            # Try Pillow
+            image = Image.open(BytesIO(mem)).convert("RGBA")
+            array = GLib.Bytes.new(image.tobytes())
+            width, height = image.size
+            pixbuf = GdkPixbuf.Pixbuf.new_from_bytes(
+                array, GdkPixbuf.Colorspace.RGB, True,
+                8, width, height, width * 4)
 
-                pixbuf.save_to_callback(cb, "jpeg", {"quality": "100"})
-            except Exception as e:
-                if output:
-                    output.close()
-                log.info("Failed to load image using gdk pixbuf, "
-                         "ignoring image.")
-                log.debug(e)
-                return ('', '')
-
-        mem = output.getvalue()
-        output.close()
         try:
             self._create_path(os.path.dirname(thumbpath))
-            self._write_file(thumbpath, mem)
-        except Exception as e:
+            thumbnail = pixbuf
+            if isinstance(pixbuf, GdkPixbuf.PixbufAnimation):
+                if size < pixbuf.get_width() or size < pixbuf.get_height():
+                    resize_gif(mem, thumbpath, (size, size))
+                    thumbnail = self._load_thumbnail(thumbpath)
+                else:
+                    self._write_file(thumbpath, mem)
+            else:
+                width, height = self._get_thumbnail_size(pixbuf, size)
+                thumbnail = pixbuf.scale_simple(
+                    width, height, GdkPixbuf.InterpType.BILINEAR)
+                thumbnail.savev(thumbpath, 'png', [], [])
+        except Exception as error:
             dialogs.ErrorDialog(
                 _('Could not save file'),
                 _('Exception raised while saving thumbnail '
                   'for image file (see error log for more '
                   'information)'),
-                transient_for=self.chat_control.parent_win.window)
-            log.error(str(e))
-        return (mem, alt)
+                transient_for=app.app.get_active_window())
+            log.exception(error)
+            return
+        return thumbnail
 
-    def _load_thumbnail(self, thumbpath):
-        with open(thumbpath, 'rb') as f:
-            mem = f.read()
-            f.closed
-        return (mem, '')
+    @staticmethod
+    def _get_thumbnail_size(pixbuf, size):
+        # Calculates the new thumbnail size while preserving the aspect ratio
+        image_width = pixbuf.get_width()
+        image_height = pixbuf.get_height()
 
-    def _write_file(self, path, data):
-        log.info("Writing '%s' of size %d..." % (path, len(data)))
+        if image_width > image_height:
+            if image_width > size:
+                image_height = int(size / float(image_width) * image_height)
+                image_width = int(size)
+        else:
+            if image_height > size:
+                image_width = int(size / float(image_height) * image_width)
+                image_height = int(size)
+
+        return image_width, image_height
+
+    @staticmethod
+    def _load_thumbnail(thumbpath):
+        ext = os.path.splitext(thumbpath)[1]
+        if ext == '.gif':
+            return GdkPixbuf.PixbufAnimation.new_from_file(thumbpath)
+        return GdkPixbuf.Pixbuf.new_from_file(thumbpath)
+
+    @staticmethod
+    def _write_file(path, data):
+        log.info("Writing '%s' of size %d...", path, len(data))
         try:
             with open(path, "wb") as output_file:
                 output_file.write(data)
                 output_file.closed
         except Exception as e:
-            log.error("Failed to write file '%s'!" % path)
+            log.error("Failed to write file '%s'!", path)
             raise
 
-    def _update_img(self, (mem, alt), url, repl_start, repl_end,
+    def _update_img(self, pixbuf, url, repl_start, repl_end,
                     filepath, encrypted):
-        if mem:
-            try:
-                urlparts = urlparse(url)
-                filename = os.path.basename(urlparts.path)
-                eb = gtk.EventBox()
-                eb.connect('button-press-event', self.on_button_press_event,
-                           filepath, filename, url, encrypted)
-                eb.connect('enter-notify-event', self.on_enter_event)
-                eb.connect('leave-notify-event', self.on_leave_event)
-
-                # this is threadsafe
-                # (gtk textview is NOT threadsafe by itself!!)
-                def add_to_textview():
-                    try:        # textview closed in the meantime etc.
-                        buffer_ = repl_start.get_buffer()
-                        iter_ = buffer_.get_iter_at_mark(repl_start)
-                        buffer_.insert(iter_, "\n")
-                        anchor = buffer_.create_child_anchor(iter_)
-                        # Use url as tooltip for image
-                        img = TextViewImage(anchor, url)
-
-                        loader = gtk.gdk.PixbufLoader()
-                        loader.write(mem)
-                        loader.close()
-                        pixbuf = loader.get_pixbuf()
-                        img.set_from_pixbuf(pixbuf)
-
-                        eb.add(img)
-                        eb.show_all()
-                        self.textview.tv.add_child_at_anchor(eb, anchor)
-                        buffer_.delete(iter_,
-                                       buffer_.get_iter_at_mark(repl_end))
-                    except:
-                        pass
-                    return False
-                # add to mainloop --> make call threadsafe
-                gobject.idle_add(add_to_textview)
-            except Exception:
-                # URL is already displayed
-                log.error('Could not display image for URL: %s'
-                          % url)
-                raise
-        else:
+        if pixbuf is None:
             # If image could not be downloaded, URL is already displayed
-            log.error('Could not download image for URL: %s -- %s'
-                      % (url, alt))
+            log.error('Could not download image for URL: %s', url)
+            return
 
-    def _check_mime_size(self, (file_mime, file_size),
+        urlparts = urlparse(url)
+        filename = os.path.basename(urlparts.path)
+        event_box = Gtk.EventBox()
+        event_box.connect('button-press-event', self.on_button_press_event,
+                          filepath, filename, url, encrypted)
+        event_box.connect('enter-notify-event', self.on_enter_event)
+        event_box.connect('leave-notify-event', self.on_leave_event)
+
+        def add_to_textview():
+            try:
+                at_end = self.textview.at_the_end()
+
+                buffer_ = repl_start.get_buffer()
+                iter_ = buffer_.get_iter_at_mark(repl_start)
+
+                anchor = buffer_.create_child_anchor(iter_)
+
+                if isinstance(pixbuf, GdkPixbuf.PixbufAnimation):
+                    image = Gtk.Image.new_from_animation(pixbuf)
+                else:
+                    image = Gtk.Image.new_from_pixbuf(pixbuf)
+                event_box.add(image)
+                event_box.show_all()
+                self.textview.tv.add_child_at_anchor(event_box, anchor)
+                buffer_.delete(iter_,
+                               buffer_.get_iter_at_mark(repl_end))
+
+                if at_end:
+                    GLib.idle_add(self.textview.scroll_to_end_iter)
+            except Exception as ex:
+                log.exception("Exception while loading %s: %s", url, ex)
+            return False
+        # add to mainloop --> make call threadsafe
+        GLib.idle_add(add_to_textview)
+
+    def _check_mime_size(self, tuple_arg,
                          url, repl_start, repl_end, filepaths,
                          key, iv, encrypted):
+        file_mime, file_size = tuple_arg
         # Check if mime type is acceptable
         if file_mime == '' and file_size == 0:
             log.info("Failed to load HEAD Request for URL: '%s'"
-                     "(see debug log for more info)" % url)
+                     "(see debug log for more info)", url)
             # URL is already displayed
             return
         if file_mime.lower() not in ACCEPTED_MIME_TYPES:
-            log.info("Not accepted mime type '%s' for URL: '%s'"
-                     % (file_mime.lower(), url))
+            log.info("Not accepted mime type '%s' for URL: '%s'",
+                     file_mime.lower(), url)
             # URL is already displayed
             return
         # Check if file size is acceptable
-        if file_size > self.plugin.config['MAX_FILE_SIZE'] or file_size == 0:
-            log.info("File size (%s) too big or unknown (zero) for URL: '%s'"
-                     % (str(file_size), url))
+        max_size = int(self.plugin.config['MAX_FILE_SIZE'])
+        if file_size > max_size or file_size == 0:
+            log.info("File size (%s) too big or unknown (zero) for URL: '%s'",
+                     file_size, url)
             # URL is already displayed
             return
 
         attributes = {'src': url,
-                      'max_size': self.plugin.config['MAX_FILE_SIZE'],
+                      'verify': self.plugin.config['VERIFY'],
+                      'max_size': max_size,
                       'filepaths': filepaths,
                       'key': key,
                       'iv': iv}
 
-        gajim.thread_interface(
+        app.thread_interface(
             self._download_image, [self.textview.account,
                                    attributes, encrypted],
             self._update_img, [url, repl_start, repl_end,
@@ -418,17 +436,17 @@ class Base(object):
                 _('Could not save file'),
                 _('Exception raised while saving image file'
                   ' (see error log for more information)'),
-                transient_for=self.chat_control.parent_win.window)
+                transient_for=app.app.get_active_window())
             log.error(str(e))
 
         # Create thumbnail, write it to harddisk and return it
-        return self._save_thumbnail(thumbpath, (mem, alt))
+        return self._save_thumbnail(thumbpath, mem)
 
     def _create_path(self, folder):
         if os.path.exists(folder):
             return
         log.debug("creating folder '%s'" % folder)
-        os.mkdir(folder, 0700)
+        os.mkdir(folder, 0o700)
 
     def _aes_decrypt_fast(self, key, iv, payload):
         # Use AES128 GCM with the given key and iv to decrypt the payload.
@@ -444,28 +462,8 @@ class Base(object):
             backend=be).decryptor()
         return decryptor.update(data) + decryptor.finalize()
 
-    def _get_pixbuf_of_size(self, pixbuf, size):
-        # Creates a pixbuf that fits in the specified square of sizexsize
-        # while preserving the aspect ratio
-        # Returns tuple: (scaled_pixbuf, actual_width, actual_height)
-        image_width = pixbuf.get_width()
-        image_height = pixbuf.get_height()
-
-        if image_width > image_height:
-            if image_width > size:
-                image_height = int(size / float(image_width) * image_height)
-                image_width = int(size)
-        else:
-            if image_height > size:
-                image_width = int(size / float(image_height) * image_width)
-                image_height = int(size)
-
-        crop_pixbuf = pixbuf.scale_simple(image_width, image_height,
-                                          gtk.gdk.INTERP_BILINEAR)
-        return (crop_pixbuf, image_width, image_height)
-
     def make_rightclick_menu(self, event, data):
-        xml = gtk.Builder()
+        xml = Gtk.Builder()
         xml.set_translation_domain('gajim_plugins')
         xml.add_from_file(self.plugin.local_file_path('context_menu.ui'))
         menu = xml.get_object('context_menu')
@@ -480,11 +478,11 @@ class Base(object):
             xml.get_object('open_file_in_browser_menuitem')
         extras_separator = \
             xml.get_object('extras_separator')
-        
+
         if data["encrypted"]:
             open_link_in_browser_menuitem.hide()
-        if gajim.config.get('autodetect_browser_mailer') \
-                or gajim.config.get('custombrowser') == '':
+        if app.config.get('autodetect_browser_mailer') \
+                or app.config.get('custombrowser') == '':
             extras_separator.hide()
             open_file_in_browser_menuitem.hide()
 
@@ -513,6 +511,7 @@ class Base(object):
     def on_save_as_menuitem_activate(self, menu, data):
         filepath = data["filepath"]
         original_filename = data["original_filename"]
+
         def on_continue(response, target_path):
             if response < 0:
                 return
@@ -552,11 +551,11 @@ class Base(object):
 
         dialog = dialogs.FileChooserDialog(
             title_text=_('Save Image as...'),
-            action=gtk.FILE_CHOOSER_ACTION_SAVE,
-            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                     gtk.STOCK_SAVE, gtk.RESPONSE_OK),
-            default_response=gtk.RESPONSE_OK,
-            current_folder=gajim.config.get('last_save_dir'),
+            action=Gtk.FileChooserAction.SAVE,
+            buttons=(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                     Gtk.STOCK_SAVE, Gtk.ResponseType.OK),
+            default_response=Gtk.ResponseType.OK,
+            current_folder=app.config.get('last_save_dir'),
             on_response_ok=on_ok,
             on_response_cancel=on_cancel)
 
@@ -566,8 +565,9 @@ class Base(object):
 
     def on_copy_link_location_menuitem_activate(self, menu, data):
         url = data["url"]
-        clipboard = gtk.Clipboard()
-        clipboard.set_text(url)
+        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+        clipboard.set_text(url, -1)
+        clipboard.store()
 
     def on_open_link_in_browser_menuitem_activate(self, menu, data):
         url = data["url"]
@@ -577,24 +577,24 @@ class Base(object):
                 _('You cannot open encrypted files in your '
                   'browser directly. Try "Open Downloaded File '
                   'in Browser" instead.'),
-                transient_for=self.chat_control.parent_win.window)
+                transient_for=app.app.get_active_window())
         else:
             helpers.launch_browser_mailer('url', url)
-        
+
     def on_open_file_in_browser_menuitem_activate(self, menu, data):
         if os.name == "nt":
             filepath = "file://" + os.path.abspath(data["filepath"])
         else:
             filepath = "file://" + data["filepath"]
-        if gajim.config.get('autodetect_browser_mailer') \
-                or gajim.config.get('custombrowser') == '':
+        if app.config.get('autodetect_browser_mailer') \
+                or app.config.get('custombrowser') == '':
             dialogs.ErrorDialog(
                 _('Cannot open downloaded file in browser'),
                 _('You have to set a custom browser executable '
                   'in your gajim settings for this to work.'),
-                transient_for=self.chat_control.parent_win.window)
+                transient_for=app.app.get_active_window())
             return
-        command = gajim.config.get('custombrowser')
+        command = app.config.get('custombrowser')
         command = helpers.build_command(command, filepath)
         try:
             helpers.exec_command(command)
@@ -605,12 +605,12 @@ class Base(object):
     # mouse enter the eventbox with the image
     def on_enter_event(self, eb, event):
         self.textview.tv.get_window(
-            gtk.TEXT_WINDOW_TEXT).set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2))
+            Gtk.TextWindowType.TEXT).set_cursor(Gdk.Cursor(Gdk.CursorType.HAND2))
 
     # Change mouse pointer to default when mouse leaves the eventbox
     def on_leave_event(self, eb, event):
         self.textview.tv.get_window(
-            gtk.TEXT_WINDOW_TEXT).set_cursor(gtk.gdk.Cursor(gtk.gdk.XTERM))
+            Gtk.TextWindowType.TEXT).set_cursor(Gdk.Cursor(Gdk.CursorType.XTERM))
 
     def on_button_press_event(self, eb, event, filepath,
                               original_filename, url, encrypted):
@@ -619,77 +619,17 @@ class Base(object):
                 "url": url,
                 "encrypted": encrypted}
         # left click
-        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
+        if event.type == Gdk.EventType.BUTTON_PRESS and event.button == 1:
             method = getattr(self, "on_"
                              + self.plugin.config['LEFTCLICK_ACTION']
                              + "_activate")
             method(event, data)
         # right klick
-        elif event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
+        elif event.type == Gdk.EventType.BUTTON_PRESS and event.button == 3:
             menu = self.make_rightclick_menu(event, data)
             # menu.attach_to_widget(self.tv, None)
-            menu.popup(None, None, None, event.button, event.time)
+            # menu.popup(None, None, None, event.button, event.time)
+            menu.popup_at_pointer(event)
 
     def disconnect_from_chat_control(self):
         pass
-
-
-class UrlImagePreviewPluginConfigDialog(GajimPluginConfigDialog):
-    max_file_size = [262144, 524288, 1048576, 5242880, 10485760]
-    leftclick_action = ['open_menuitem', 'save_as_menuitem', 'copy_link_location_menuitem',
-                        'open_link_in_browser_menuitem', 'open_file_in_browser_menuitem']
-
-    def init(self):
-        self.GTK_BUILDER_FILE_PATH = self.plugin.local_file_path(
-            'config_dialog.ui')
-        self.xml = gtk.Builder()
-        self.xml.set_translation_domain('gajim_plugins')
-        self.xml.add_objects_from_file(self.GTK_BUILDER_FILE_PATH, [
-            'vbox1', 'liststore1', 'liststore2'])
-        self.preview_size_spinbutton = self.xml.get_object('preview_size')
-        self.preview_size_spinbutton.get_adjustment().set_all(20, 10, 512, 1,
-                                                              10, 0)
-        self.max_size_combobox = self.xml.get_object('max_size_combobox')
-        self.leftclick_action_combobox = self.xml.get_object('leftclick_action_combobox')
-        vbox = self.xml.get_object('vbox1')
-        self.child.pack_start(vbox)
-
-        self.xml.connect_signals(self)
-
-    def on_run(self):
-        self.preview_size_spinbutton.set_value(self.plugin.config[
-            'PREVIEW_SIZE'])
-        value = self.plugin.config['MAX_FILE_SIZE']
-        if value:
-            # this fails if we upgrade from an old version
-            # which has other file size values than we have now
-            try:
-                self.max_size_combobox.set_active(
-                    self.max_file_size.index(value))
-            except:
-                pass
-        else:
-            self.max_size_combobox.set_active(-1)
-        
-        value = self.plugin.config['LEFTCLICK_ACTION']
-        if value:
-            # this fails if we upgrade from an old version
-            # which has other file size values than we have now
-            try:
-                self.leftclick_action_combobox.set_active(
-                    self.leftclick_action.index(value))
-            except:
-                pass
-        else:
-            self.leftclick_action_combobox.set_active(0)
-
-    def preview_size_value_changed(self, spinbutton):
-        self.plugin.config['PREVIEW_SIZE'] = spinbutton.get_value()
-
-    def max_size_value_changed(self, widget):
-        self.plugin.config['MAX_FILE_SIZE'] = self.max_file_size[
-            self.max_size_combobox.get_active()]
-    
-    def leftclick_action_changed(self, widget):
-        self.plugin.config['LEFTCLICK_ACTION'] = self.leftclick_action[
-            self.leftclick_action_combobox.get_active()]
